#ifndef __ARGPARSER_H__
#define __ARGPARSER_H__

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <getopt.h>

#define VERSION "1.2"

/*
 * Cmdline options
 */
struct {
    int inject;   // inject mode
    int sniff;    // sniff mode
    int raw;      // raw mode
    char *device; // interface
    char *filter; // custom pcap filter
    int count;    // number of packets to capture
    int count_on; // enable count
    int no_cksum; // disable packet checksum
    int no_size;  // disable packet size
} options;

/*
 * Program usage template
 */
const char usage_template[] =
    "HexInject " VERSION " [hexadecimal packet injector/sniffer]\n"
    "written by: Emanuele Acri <crossbower@gmail.com>\n\n"
    "Usage:\n"
    "   hexinject <options>\n"
    "Options:\n"
    "  -s sniff mode\n"
    "  -p inject mode\n"
    "  -r raw mode (instead of the default hexadecimal mode)\n"
    "  -i device: network device to use\n"
    "  -f filter: custom pcap filter\n"
    "  -c number of packets to capture\n"
    "  -C disable automatic packet checksum\n"
    "  -S disable automatic packet size\n"
    "  -h help screen\n";

/*
 * Program usage
 */
void usage(FILE *out, const char *error)
{
    fputs(usage_template, out);

    if(error)
        fputs(error, out);

    exit(1);
}

/*
 * Parser for command line options
 * See getopt(3)...
 */
int parseopt(int argc, char **argv)
{
    char *c=NULL, *x=NULL;
    char ch;
    
    // cleaning
    memset(&options, 0, sizeof(options));
    
    
    const char *shortopt = "spri:f:c:Ch"; // short options
    
    while ((ch = getopt (argc, argv, shortopt)) != -1) {
        switch (ch) {
        
            case 's': // sniff mode
                options.sniff = 1;
                break;
            
            case 'p': // inject mode
                options.inject = 1;
                break;
                
            case 'r': // raw mode
                options.raw = 1;
                break;  
            
            case 'i': // interface
                options.device = optarg;
                break;
            
            case 'f': // custom filter
                options.filter = optarg;
                break;
                
            case 'c': // packet count
                options.count = atoi(optarg);
                options.count_on = 1;
                break;

            case 'C': // disable packet checksum
                options.no_cksum = 1;
                break;

            case 'S': // disable packet size
                options.no_size = 1;
                break;
            
            case 'h': //help
                usage(stdout, NULL);

            case '?':
            default:
                usage(stderr, NULL);
        }
    }
    
    // check mode
    if ( !options.inject && !options.sniff ) {
        usage(stderr, "\nError: no mode selected, see -s and -p options...\n");
    }
    
    // check interface
    if ( options.device == NULL ) {
        usage(stderr, "\nError: no interface selected, see -i option...\n");
    }
	
    return 1;
}

#endif /* __ARGPARSER_H__ */

